import {
  ConflictException,
  ForbiddenException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { CarreraPrograma } from './entity/carrera-programa.entity';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionCarreraService } from '../institucion-carrera/institucion-carrera.service';
import { InstitucionProgramaService } from '../institucion-programa/institucion-programa.service';
import { InstitucionUsuarioService } from '../institucion-usuario/institucion-usuario.service';

@Injectable()
export class CarreraProgramaService {
  constructor(
    @InjectRepository(CarreraPrograma)
    private repository: Repository<CarreraPrograma>,
    private institucionService: InstitucionService,
    private institucionCarreraService: InstitucionCarreraService,
    private institucionProgramaService: InstitucionProgramaService,
    private institucionUsuarioService: InstitucionUsuarioService,
  ) {}

  async create(
    admin: Operador,
    id_institucion_carrera: number,
    id_programa: number,
  ) {
    const institucionCarrera = await this.institucionCarreraService.findById(
      id_institucion_carrera,
    );
    const programa = await this.institucionProgramaService.findProgramaById(
      id_programa,
    );

    // Valido que el admin que esta realizando esta acción le pertenezca la carrera
    if (
      admin.institucion.id_institucion !=
      institucionCarrera.institucion.id_institucion
    )
      throw new ForbiddenException(
        'Esta carrera no pertenece a tu institución.',
      );
    // Busco un registro que ya contenga la info mandada
    return this.repository
      .findOne({
        select: { id_carrera_programa: true },
        where: { institucionCarrera, programa },
      })
      .then((existeCarretaPrograma) => {
        // Si lo encuentro saco error
        if (existeCarretaPrograma)
          throw new ConflictException(
            'Ya se asignó este programa a esta carrera.',
          );
        // Guardar registro
        return this.repository.save(
          this.repository.create({
            institucionCarrera,
            programa,
          }),
        );
      })
      .then((_) => ({
        message: 'Se asignó correctamente el programa a la carrera.',
      }));
  }

  delete(admin: Operador, id_carrera_programa: number) {
    return this.findById(id_carrera_programa)
      .then((carreraPrograma) => {
        // Valido que la asociación entre una carrera y un programa le
        // pertenezcan al admin que realiza esta acción
        if (
          admin.institucion.id_institucion !=
          carreraPrograma.institucionCarrera.institucion.id_institucion
        )
          throw new ForbiddenException(
            'No puedes eliminar esta asociación porque no pertenece a tu institución.',
          );
        // Elimino
        return this.repository.remove(carreraPrograma);
      })
      .then((_) => ({
        message: 'Se eliminó correctamente la asignación del programa.',
      }));
  }

  findById(id_carrera_programa: number) {
    return this.repository
      .findOne({ where: { id_carrera_programa } })
      .then((carreraPrograma) => {
        if (!carreraPrograma)
          throw new NotFoundException('No existe este id carrera programa.');
        return carreraPrograma;
      });
  }

  findByIdInstitucion(
    usuarioOperador: Operador | Usuario,
    id_institucion: number,
  ) {
    return this.institucionService
      .findById(id_institucion)
      .then(async (institucion) => {
        // Si es un Operador
        if (usuarioOperador instanceof Operador) {
          // Valido que la info le pertenezca al operador que la solicita
          if (
            usuarioOperador.institucion.id_institucion !=
            institucion.id_institucion
          )
            throw new ForbiddenException(
              'No puedes acceder a esta información porque no le pertenece a tu institución.',
            );
        } else
          await this.institucionUsuarioService
            .findByIdUsuarioIdInstitucion(usuarioOperador, institucion)
            .then((institucionUsuario) => {
              if (!institucionUsuario)
                throw new ForbiddenException(
                  'No puedes acceder a esta información porque no perteneces a esta institución.',
                );
            });
        return this.repository.find({
          join: {
            alias: 'cp',
            innerJoinAndSelect: {
              ic: 'cp.institucionCarrera',
              c: 'ic.carrera',
              i: 'ic.institucion',
            },
          },
          where: { institucionCarrera: { institucion } },
        });
      });
  }
}
